$(document).ready(async function () {

  $('#tutorial').on('click', function () {
    $(document).ready(async function () {
      const driver = new Driver({
        doneBtnText: 'Sair',
        closeBtnText: 'Fechar',
        nextBtnText: 'Próximo',
        prevBtnText: 'Anterior',
        allowClose: false
      });
      driver.defineSteps([{
        element: '#btnFiltros',
        popover: {
          title: 'Botão Filtros',
          description: 'Clique aqui para abrir os filtros e realizar uma consulta.',
          position: 'right'
        }
      },
      {
        element: '#btnImprimir',
        popover: {
          title: 'Botão Imprimir',
          description: 'Clique aqui para imprimir um relatório.',
          position: 'right'
        }
      },
      {
        element: '#btnRelCliente',
        popover: {
          title: 'Tabela de cliente',
          description: 'Tabela onde os dados consultados serão listados.',
          position: 'right'
        }
      },
      ]);
      driver.start();
    });
  });

  $('#btnFiltros').on('click', async function () {
    $('#modal-filtros').modal('show');
  });

  $('#txtUF').listaFiltros();
  $('#txtCliente').listaFiltros();

  //Buscar automaticamente ao digitar
  $('#txtCliente').autocompleta(3, `JSON={ "tabela":"ENTIDADE", "camposSelect":[ "CODCLI CHAVE", "NOME DESCRICAO"], "where": null}`);
  $('#txtCliente').autocompleta(3, `JSON={ "tabela":"entidade", "camposSelect":["CODCLI CHAVE, NOME ||' - '|| CNPJ DESCRICAO, CNPJ"], "where": [] }`, ['#txtCNPJ'], ['CNPJ'], ['CNPJ']);
  $('#txtCliente').autocompleta(3, `JSON={ "tabela":"entidade", "camposSelect":["CODCLI CHAVE, NOME DESCRICAO, CNPJ"], "where": [] }`, ['#txt-cnpj'], ['CNPJ'], ['CNPJ']);
  $('#txtUF').autocompleta(1, `JSON={ "tabela":"TABUF", "camposSelect":[ "CODIGO CHAVE", "DESCRICAO DESCRICAO"], "where": null}`);

  $('#btnCliente').pesquisa_serverside(['#txtCliente', '#txtCNPJ'], ['CODCLI', 'CNPJ'], ['NOME', 'CNPJ'], '{"tabela":"entidade", "camposSelect":[ "CODCLI", "NOME", "FANTASIA", "CNPJ" ], "where": null}', 'Pesquisa Cliente');
  $("#btnUF").pesquisa("#txtUF", "CODIGO", "DESCRICAO", `/sisplan/funcoes/v1/pesquisa?JSON={ "tabela":"TABUF", "camposSelect":[ "CODIGO", "DESCRICAO"] }`, "Pesquisa UF", "TABUF");

  //limpar os campos
  $('#btnLimpaCliente').on('click', function () {
    $('#listaCliente').val('');
  });

  $('#btnLimpaUF').on('click', function () {
    $('#listaUF').val('');
  });

  //limpas todos os campos
  $('#btnLimparFiltros').on('click', async function () {
    $('#dtCadastroDe').val('1200-01-01');
    $('#dtCadastroAte').val('2500-12-31');
    $('#dtNascimentoDe').val('1200-01-01');
    $('#dtNascimentoAte').val('2500-12-31');
    $('#listaCliente').val('');
    $('#listaUF').val('');
  });

  async function retornaFiltros() {
    const sCadastroDe = $('#dtCadastroDe').val() != '1200-01-01' ? $('#dtCadastroDe').val() : '';
    const sCadastroAte = $('#dtCadastroAte').val() != '2500-12-31' ? $('#dtCadastroAte').val() : '';
    const sNascimentoDe = $('#dtNascimentoDe').val() != '1200-01-01' ? $('#dtNascimentoDe').val() : '';
    const sNascimentoAte = $('#dtNascimentoAte').val() != '2500-12-31' ? $('#dtNascimentoAte').val() : '';
    const sCliente = await formataListas($('#listaCliente').val());
    const sUFs = await formataListas($('#listaUF').val());
    const sOrdenacao = $('#txtOrdenacao').val();

    const retorno =
      `CADASTRO_DE=${sCadastroDe}` +
      `&CADASTRO_ATE=${sCadastroAte}` +
      `&NASCIMENTO_DE=${sNascimentoDe}` +
      `&NASCIMENTO_ATE=${sNascimentoAte}` +
      `&CLIENTE=${sCliente}` +
      `&UFS=${sUFs}` +
      `&MES=${pegaValor('#txtMes')}` +
      `&ORDENACAO=${sOrdenacao}`;

    return retorno;
  }

  $('#btnImprimir').on('click', async function () {
    try {
      $.LoadingOverlay('show');
      try {
        const url = `/sisplan/impressao/v1/imprimirrelaniversario?`;
        const filtros = await retornaFiltros();
        const bImprimeAutomatico = await verificaImprimirAutomatico('RelAniversario');
        const arquivo = await GeraRelatorio(`${url}${filtros}&`, 'GET', 100000, false, 'RelAniversario');
        if (arquivo != undefined) {
          if (!bImprimeAutomatico) {
            window.open(`${BASE_URI}/relatorios_api/pdf/${arquivo}`, '_blank');
          }
          await limparRelatorios();
        }

        $('#btnLimparFiltros').trigger('click');

      } catch (error) {
        console.error(error);
      }
    } finally {
      $.LoadingOverlay('hide');
    }
  });

  $('#btnConsultar').on('click', async function () {
    if ($('#tabelaAniversario')[0].rows.length > 0) {
      await limpaTabela();
    }

    const where = await montaWhere();
    const campos = await montaCamposSelect();
    const ligacao = await montaLigacoes();
    const orderBy = await montaOrderBy($('#txtOrdenacao').val());

    let dataSetCols = [];

    try {
      try {
        $.LoadingOverlay("show");

        for (let i = 0; i < campos.colunas.length; i++) {
          dataSetCols.push({
            'title': campos.colunas[i],
            'data': i
          });
        }

        const ajaxData = {
          "EMPRESA": getCookie('empresa'),
          "JSON": `{"tabela":"ENTIDADE", "camposSelect": ${campos.camposSelect}, "leftJoin": [${ligacao}], "where": ["1=1 ${where}"] ${orderBy != '' ? `, "orderby": ["${orderBy}"]` : ''}}`,
          "CLIENTE": 'N',
        };

        $('#tabelaAniversario').DataTable({
          paging: true,
          lengthChange: false,
          filter: false,
          info: true,
          order: false,
          autoWidth: true,
          destroy: true,
          serverSide: true,
          processing: true,
          ajax: {
            url: `${getCookie('protocolo')}://${getCookie('ip_api')}/sisplan/funcoes/v1/pesquisadatatable`,
            type: "GET",
            headers: {
              'Authorization': BASIC_AUTH
            },
            data: ajaxData,
          },
          "language": {
            "sEmptyTable": "Nenhuma consulta realizada",
            "sInfo": "_TOTAL_ registros",
            "sInfoEmpty": " 0 registros",
            "sInfoFiltered": "(Filtrados de _MAX_ registros)",
            "sInfoPostFix": "",
            "sInfoThousands": ".",
            "sLengthMenu": "_MENU_ resultados",
            "sLoadingRecords": "Carregando...",
            "sProcessing": "Processando...",
            "sZeroRecords": "Nenhuma consulta realizada",
            "sSearch": "Pesquisar",
            "oPaginate": {
              "sNext": "Próximo",
              "sPrevious": "Anterior",
              "sFirst": "Primeiro",
              "sLast": "Último"
            },
            "oAria": {
              "sSortAscending": ": Ordenar colunas de forma ascendente",
              "sSortDescending": ": Ordenar colunas de forma descendente"
            }
          },
          columns: dataSetCols
        });


      } catch (error) {
        console.error(error);
        msgErro('Não foi possível buscar os dados.');
      }
    } finally {
      $.LoadingOverlay("hide");
      $('#modal-filtros').modal('hide');
    }

  });

  async function montaWhere() {
    let where = '';
    where += $('#dtNascimentoDe').val() != '' ? ` AND ENTIDADE.DT_NASC >= '${$('#dtNascimentoDe').val()}'` : '';
    where += $('#dtNascimentoAte').val() != '' ? ` AND ENTIDADE.DT_NASC <= '${$('#dtNascimentoAte').val()}'` : '';
    where += $('#dtCadastroDe').val() != '' ? $('#dtCadastroDe').val() != '1200-01-01' ? ` AND ENTIDADE.DATA_CAD >= '${$('#dtCadastroDe').val()}'` : '' : '';
    where += $('#dtCadastroAte').val() != '' ? $('#dtCadastroAte').val() != '2500-12-31' ? ` AND ENTIDADE.DATA_CAD <= '${$('#dtCadastroAte').val()}'` : '' : '';
    where += $('#listaCliente').val() != '' ? ` AND ENTIDADE.CODCLI in (${await formataListas($('#listaCliente').val())})` : '';
    where += $('#listaUF').val() != '' ? ` AND TABUF.CODIGO in (${await formataListas($('#listaUF').val())})` : '';
    where += pegaValor('#txtMes') != '' ? ` AND UDF_MONTH(ENTIDADE.DT_NASC) = ${pegaValor('#txtMes')}` : '';

    return where;

  }

  async function montaLigacoes() {

    ligacao = `{"tabela": "CADCEP", "condicao": "CADCEP.CEP = ENTIDADE.CEP", "alias": "CADCEP"},` +
      `{"tabela": "CIDADE", "condicao": "CADCEP.CODMUN = CIDADE.CODIGO", "alias": "CIDADE"},` +
      `{"tabela": "TABUF", "condicao": "TABUF.CODIGO = CIDADE.COD_UF", "alias": "TABUF"}`;

    return ligacao;
  }

  async function montaCamposSelect() {
    let camposSelect = [];
    let colunas = [];


    camposSelect = `["ENTIDADE.CODCLI", "ENTIDADE.NOME","ENTIDADE.DT_NASC","ENTIDADE.DATA_CAD", "ENTIDADE.TELEFONE"]`;

    colunas = ["CODCLI", "NOME", "DT_NASC", "DATA_CAD", "TELEFONE"];

    return objeto = {
      camposSelect: camposSelect,
      colunas: colunas
    };

  }

  async function montaOrderBy(ordenacao) {
    let order = '';

    switch (ordenacao) {
      case '0':
        order = 'ENTIDADE.CODCLI';
        break;
      case '1':
        order = 'ENTIDADE.NOME';
        break;
      case '2':
        order = 'UDF_DAY(ENTIDADE.DT_NASC)';
        break;
      case '3':
        order = 'ENTIDADE.DATA_CAD';
        break;
      default:
        break;
    }

    return order;
  }

  async function limpaTabela() {
    $('#tabelaAniversario').DataTable().destroy().draw(false);
    $('#tabelaAniversario').html('<thead></thead><tbody></tbody>');
  }

});